#!/usr/bin/perl

use strict;
use integer;

my($infile,$outfile,$depth,$width) = @ARGV;

my $bytesperword = ($width+7) >> 3;

open(my $in,  '<:raw', $infile) or die;
my $data;
my $dlen = read($in,$data,$depth*$bytesperword);
close($in);

open(my $out, '>', $outfile) or die;

printf $out "-- generated from %s\n", $infile;
printf $out "DEPTH = %d;\n", $depth;
printf $out "WIDTH = %d;\n", $width;
printf $out "ADDRESS_RADIX = HEX;\n";
printf $out "DATA_RADIX = HEX;\n";
printf $out "CONTENT BEGIN\n";

my $adig = 1;
while ((1 << ($adig*4)) < $depth) {
    $adig++;
}

my $addr;
for ($addr = 0; $addr < $depth; $addr++) {
    my $offs = $addr*$bytesperword;
    last if ($offs >= $dlen);
    printf $out "%0${adig}X : ", $addr;
    # Emitting data byte by byte avoids need for bigint
    for (my $b = $bytesperword-1; $b >= 0; $b--) {
	if ($offs+$b < $dlen) {
	    printf $out "%02X", unpack("C", substr($data,$offs+$b,1));
	}
    }
    printf $out ";\n";
}

if ($addr < $depth) {
    printf $out "[%0${adig}X..%0${adig}X] : 00;\n", $addr, $depth-1;
}

printf $out "END;\n";
close($out);

exit 0;
    
